@@ -8,9 +8,12 @@
 // SPDX-License-Identifier: EPL-2.0
 //
 
+//! Implementation of the 'db' subcommand
+
 use std::io::Write;
 use std::path::PathBuf;
 use std::process::Command;
+use std::str::FromStr;
 
 use anyhow::Context;
 use anyhow::Error;
@@ -24,43 +27,52 @@ use diesel::JoinOnDsl;
 use diesel::QueryDsl;
 use diesel::RunQueryDsl;
 use itertools::Itertools;
+use log::debug;
 use log::info;
+use log::trace;
 
+use crate::commands::util::get_date_filter;
 use crate::config::Configuration;
 use crate::db::models;
 use crate::db::DbConnectionConfig;
-use crate::log::LogItem;
+use crate::log::JobResult;
 use crate::package::Script;
 use crate::schema;
 
+diesel_migrations::embed_migrations!("migrations");
+
 /// Implementation of the "db" subcommand
 pub fn db(
-    db_connection_config: DbConnectionConfig,
+    db_connection_config: DbConnectionConfig<'_>,
     config: &Configuration,
     matches: &ArgMatches,
 ) -> Result<()> {
     match matches.subcommand() {
         Some(("cli", matches)) => cli(db_connection_config, matches),
+        Some(("setup", _matches)) => setup(db_connection_config),
         Some(("artifacts", matches)) => artifacts(db_connection_config, matches),
         Some(("envvars", matches)) => envvars(db_connection_config, matches),
         Some(("images", matches)) => images(db_connection_config, matches),
+        Some(("submit", matches)) => submit(db_connection_config, matches),
         Some(("submits", matches)) => submits(db_connection_config, matches),
         Some(("jobs", matches)) => jobs(db_connection_config, matches),
         Some(("job", matches)) => job(db_connection_config, config, matches),
+        Some(("log-of", matches)) => log_of(db_connection_config, matches),
         Some(("releases", matches)) => releases(db_connection_config, config, matches),
         Some((other, _)) => Err(anyhow!("Unknown subcommand: {}", other)),
         None => Err(anyhow!("No subcommand")),
     }
 }
 
-fn cli(db_connection_config: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the "db cli" subcommand
+fn cli(db_connection_config: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     trait PgCliCommand {
-        fn run_for_uri(&self, dbcc: DbConnectionConfig) -> Result<()>;
+        fn run_for_uri(&self, dbcc: DbConnectionConfig<'_>) -> Result<()>;
     }
 
     struct Psql(PathBuf);
     impl PgCliCommand for Psql {
-        fn run_for_uri(&self, dbcc: DbConnectionConfig) -> Result<()> {
+        fn run_for_uri(&self, dbcc: DbConnectionConfig<'_>) -> Result<()> {
             Command::new(&self.0)
                 .arg(format!("--dbname={}", dbcc.database_name()))
                 .arg(format!("--host={}", dbcc.database_host()))
@@ -89,12 +101,12 @@ fn cli(db_connection_config: DbConnectionConfig, matches: &ArgMatches) -> Result
 
     struct PgCli(PathBuf);
     impl PgCliCommand for PgCli {
-        fn run_for_uri(&self, dbcc: DbConnectionConfig) -> Result<()> {
+        fn run_for_uri(&self, dbcc: DbConnectionConfig<'_>) -> Result<()> {
             Command::new(&self.0)
                 .arg("--host")
                 .arg(dbcc.database_host())
                 .arg("--port")
-                .arg(dbcc.database_port())
+                .arg(dbcc.database_port().to_string())
                 .arg("--username")
                 .arg(dbcc.database_user())
                 .arg(dbcc.database_name())
@@ -136,12 +148,18 @@ fn cli(db_connection_config: DbConnectionConfig, matches: &ArgMatches) -> Result
         .run_for_uri(db_connection_config)
 }
 
-fn artifacts(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
+fn setup(conn_cfg: DbConnectionConfig<'_>) -> Result<()> {
+    let conn = conn_cfg.establish_connection()?;
+    embedded_migrations::run_with_output(&conn, &mut std::io::stdout()).map_err(Error::from)
+}
+
+/// Implementation of the "db artifacts" subcommand
+fn artifacts(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     use crate::schema::artifacts::dsl;
 
     let csv = matches.is_present("csv");
-    let hdrs = crate::commands::util::mk_header(vec!["id", "path", "released", "job id"]);
-    let conn = crate::db::establish_connection(conn_cfg)?;
+    let hdrs = crate::commands::util::mk_header(vec!["Path", "Released", "Job"]);
+    let conn = conn_cfg.establish_connection()?;
     let data = matches
         .value_of("job_uuid")
         .map(uuid::Uuid::parse_str)
@@ -168,7 +186,6 @@ fn artifacts(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
                 .map(|r| r.release_date.to_string())
                 .unwrap_or_else(|| String::from("no"));
             vec![
-                format!("{}", artifact.id),
                 artifact.path,
                 rel,
                 job.uuid.to_string(),
@@ -185,16 +202,17 @@ fn artifacts(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn envvars(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the "db envvars" subcommand
+fn envvars(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     use crate::schema::envvars::dsl;
 
     let csv = matches.is_present("csv");
-    let hdrs = crate::commands::util::mk_header(vec!["id", "name", "value"]);
-    let conn = crate::db::establish_connection(conn_cfg)?;
+    let hdrs = crate::commands::util::mk_header(vec!["Name", "Value"]);
+    let conn = conn_cfg.establish_connection()?;
     let data = dsl::envvars
         .load::<models::EnvVar>(&conn)?
         .into_iter()
-        .map(|evar| vec![format!("{}", evar.id), evar.name, evar.value])
+        .map(|evar| vec![evar.name, evar.value])
         .collect::<Vec<_>>();
 
     if data.is_empty() {
@@ -206,16 +224,17 @@ fn envvars(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn images(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the "db images" subcommand
+fn images(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     use crate::schema::images::dsl;
 
     let csv = matches.is_present("csv");
-    let hdrs = crate::commands::util::mk_header(vec!["id", "name"]);
-    let conn = crate::db::establish_connection(conn_cfg)?;
+    let hdrs = crate::commands::util::mk_header(vec!["Name"]);
+    let conn = conn_cfg.establish_connection()?;
     let data = dsl::images
         .load::<models::Image>(&conn)?
         .into_iter()
-        .map(|image| vec![format!("{}", image.id), image.name])
+        .map(|image| vec![image.name])
         .collect::<Vec<_>>();
 
     if data.is_empty() {
@@ -227,58 +246,146 @@ fn images(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn submits(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the "db submit" subcommand
+fn submit(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
+    let conn = conn_cfg.establish_connection()?;
+    let submit_id = matches.value_of("submit")
+        .map(uuid::Uuid::from_str)
+        .transpose()
+        .context("Parsing submit UUID")?
+        .unwrap(); // safe by clap
+
+    let submit = models::Submit::with_id(&conn, &submit_id)
+        .with_context(|| anyhow!("Loading submit '{}' from DB", submit_id))?;
+
+    let jobs = schema::submits::table
+        .inner_join(schema::jobs::table)
+        .filter(schema::submits::uuid.eq(&submit_id))
+        .select(schema::jobs::all_columns)
+        .load::<models::Job>(&conn)
+        .with_context(|| anyhow!("Loading jobs for submit = {}", submit_id))?;
+
+    let n_jobs = jobs.len();
+    let (jobs_unknown, jobs_success, jobs_err) = {
+        let mut unkn = 0;
+        let mut succ = 0;
+        let mut err = 0;
+
+        for j in jobs.iter() {
+            match crate::log::ParsedLog::from_str(&j.log_text)?.is_successfull() {
+                JobResult::Unknown => unkn += 1,
+                JobResult::Success => succ += 1,
+                JobResult::Errored => err += 1,
+            }
+        }
+
+        (unkn, succ, err)
+    };
+
+    let out = std::io::stdout();
+    let mut outlock = out.lock();
+
+    indoc::writedoc!(outlock, r#"
+            Submit   {submit_id}
+            Date:    {submit_dt}
+            Jobs:    {n_jobs}
+            Success: {n_jobs_success}
+            Unknown: {n_jobs_unknown}
+            Errored: {n_jobs_err}
+
+        "#,
+        submit_id = submit.uuid.to_string().cyan(),
+        submit_dt = submit.submit_time.to_string().cyan(),
+        n_jobs = n_jobs.to_string().cyan(),
+        n_jobs_success = jobs_success.to_string().green(),
+        n_jobs_unknown = jobs_unknown.to_string().red(),
+        n_jobs_err = jobs_err.to_string().red(),
+    )?;
+
+    let header = crate::commands::util::mk_header(["Job", "Success", "Package", "Version", "Container", "Endpoint", "Image"].to_vec());
+    let data = jobs.iter()
+        .map(|job| {
+            let image = models::Image::fetch_for_job(&conn, job)?
+                .ok_or_else(|| anyhow!("Image for job {} not found", job.uuid))?;
+            let package = models::Package::fetch_for_job(&conn, job)?
+                .ok_or_else(|| anyhow!("Package for job {} not found", job.uuid))?;
+            let endpoint = models::Endpoint::fetch_for_job(&conn, job)?
+                .ok_or_else(|| anyhow!("Endpoint for job {} not found", job.uuid))?;
+
+            Ok(vec![
+                job.uuid.to_string().cyan(),
+                match is_job_successfull(job)? {
+                    Some(true) => "Success".green(),
+                    Some(false) => "Error".red(),
+                    None => "Unknown".yellow(),
+                },
+                package.name.cyan(),
+                package.version.cyan(),
+                job.container_hash.normal(),
+                endpoint.name.normal(),
+                image.name.normal(),
+            ])
+        })
+        .collect::<Result<Vec<Vec<colored::ColoredString>>>>()?;
+    crate::commands::util::display_data(header, data, false)
+}
+
+/// Implementation of the "db submits" subcommand
+fn submits(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     let csv = matches.is_present("csv");
-    let hdrs = crate::commands::util::mk_header(vec!["id", "time", "uuid"]);
-    let conn = crate::db::establish_connection(conn_cfg)?;
+    let limit = matches.value_of("limit").map(i64::from_str).transpose()?;
+    let hdrs = crate::commands::util::mk_header(vec!["Time", "UUID"]);
+    let conn = conn_cfg.establish_connection()?;
+
+    let query = schema::submits::table
+        .order_by(schema::submits::id.desc()); // required for the --limit implementation
+
+    let submits = if let Some(pkgname) = matches.value_of("with_pkg").map(String::from) {
+        // Get all submits which included the package, but were not necessarily made _for_ the package
+        let query = query
+            .inner_join(schema::jobs::table)
+            .inner_join(schema::packages::table.on(schema::jobs::package_id.eq(schema::packages::id)))
+            .filter(schema::packages::name.eq(&pkgname))
+            .into_boxed();
+
+        if let Some(limit) = limit {
+            query.limit(limit)
+        } else {
+            query
+        }
+        .select(schema::submits::all_columns)
+        .load::<models::Submit>(&conn)?
+    } else if let Some(pkgname) = matches.value_of("for_pkg") {
+        // Get all submits _for_ the package
+        let query = query
+            .inner_join({
+                schema::packages::table.on(schema::submits::requested_package_id.eq(schema::packages::id))
+            })
+            .filter(schema::packages::dsl::name.eq(&pkgname))
+            .into_boxed();
+
+        if let Some(limit) = limit {
+            query.limit(limit)
+        } else {
+            query
+        }
+        .select(schema::submits::all_columns)
+        .load::<models::Submit>(&conn)?
+    } else if let Some(limit) = limit {
+        query.limit(limit).load::<models::Submit>(&conn)?
+    } else {
+        query.load::<models::Submit>(&conn)?
+    };
 
     // Helper to map Submit -> Vec<String>
     let submit_to_vec = |submit: models::Submit| {
         vec![
-            format!("{}", submit.id),
             submit.submit_time.to_string(),
             submit.uuid.to_string(),
         ]
     };
 
-    // Helper to get all submits that were made _for_ a package
-    let submits_for = |pkgname: &str| {
-        schema::submits::table
-            .inner_join(schema::packages::table)
-            .filter(schema::packages::dsl::name.eq(&pkgname))
-            .select(schema::submits::all_columns)
-            .load::<models::Submit>(&conn)
-    };
-
-    let data = if let Some(pkgname) = matches.value_of("with_pkg").map(String::from) {
-        // Get all submits which included the package, but were not made _for_ the package
-        let submits_with_pkg = schema::packages::table
-            .filter(schema::packages::name.eq(&pkgname))
-            .inner_join(schema::jobs::table.inner_join(schema::submits::table))
-            .select(schema::submits::all_columns)
-            .load::<models::Submit>(&conn)?;
-
-        let submits_for_pkg = submits_for(&pkgname)?;
-
-        submits_with_pkg
-            .into_iter()
-            .chain(submits_for_pkg.into_iter())
-            .map(submit_to_vec)
-            .collect::<Vec<_>>()
-    } else if let Some(pkgname) = matches.value_of("for_pkg") {
-        // Get all submits _for_ the package
-        submits_for(pkgname)?
-            .into_iter()
-            .map(submit_to_vec)
-            .collect::<Vec<_>>()
-    } else {
-        // default: Get all submits
-        schema::submits::table
-            .load::<models::Submit>(&conn)?
-            .into_iter()
-            .map(submit_to_vec)
-            .collect::<Vec<_>>()
-    };
+    let data = submits.into_iter().rev().map(submit_to_vec).collect::<Vec<_>>();
 
     if data.is_empty() {
         info!("No submits in database");
@@ -289,119 +396,91 @@ fn submits(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn jobs(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
-    use crate::schema::jobs::dsl;
-
+/// Implementation of the "db jobs" subcommand
+fn jobs(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     let csv = matches.is_present("csv");
     let hdrs = crate::commands::util::mk_header(vec![
-        "id",
-        "submit uuid",
-        "job uuid",
-        "time",
-        "endpoint",
-        "success",
-        "package",
-        "version",
-        "Env",
+        "Submit",
+        "Job",
+        "Time",
+        "Endpoint",
+        "Success",
+        "Package",
+        "Version",
     ]);
-    let conn = crate::db::establish_connection(conn_cfg)?;
-    let env_filter_tpl = matches
-        .value_of("env_filter")
-        .map(crate::util::env::parse_to_env)
-        .transpose()?;
+    let conn = conn_cfg.establish_connection()?;
+    let older_than_filter = get_date_filter("older_than", matches)?;
+    let newer_than_filter = get_date_filter("newer_than", matches)?;
 
-    let jobs = matches
-        .value_of("submit_uuid")
-        .map(uuid::Uuid::parse_str)
-        .transpose()?
-        .map(|submit_uuid| {
-            let sel = schema::jobs::table
-                .inner_join(schema::submits::table)
-                .inner_join(schema::endpoints::table)
-                .inner_join(schema::packages::table)
-                .left_outer_join(schema::job_envs::table.inner_join(schema::envvars::table))
-                .filter(schema::submits::uuid.eq(&submit_uuid))
-                .into_boxed();
+    let mut sel = schema::jobs::table
+        .inner_join(schema::submits::table)
+        .inner_join(schema::endpoints::table)
+        .inner_join(schema::packages::table)
+        .into_boxed();
 
-            let sel = if let Some((name, val)) = env_filter_tpl.as_ref() {
+    if let Some(submit_uuid) = matches.value_of("submit_uuid").map(uuid::Uuid::parse_str).transpose()? {
+        sel = sel.filter(schema::submits::uuid.eq(submit_uuid))
+    }
+
+    // Filter for environment variables from the CLI
+    //
+    // If we get a filter for environment on CLI, we fetch all job ids that are associated with the
+    // passed environment variables and make `sel` filter for those.
+    if let Some((name, val)) = matches.value_of("env_filter").map(crate::util::env::parse_to_env).transpose()? {
+        debug!("Filtering for ENV: {} = {}", name, val);
+        let jids = schema::envvars::table
+            .filter({
                 use crate::diesel::BoolExpressionMethods;
+                schema::envvars::dsl::name.eq(name.as_ref())
+                    .and(schema::envvars::dsl::value.eq(val))
+            })
+            .inner_join(schema::job_envs::table)
+            .select(schema::job_envs::job_id)
+            .load::<i32>(&conn)?;
 
-                sel.filter({
-                    schema::envvars::dsl::name.eq(name.as_ref())
-                        .and(schema::envvars::dsl::value.eq(val))
-                })
-            } else {
-                sel
-            };
+        debug!("Filtering for these IDs (because of env filter): {:?}", jids);
+        sel = sel.filter(schema::jobs::dsl::id.eq_any(jids));
+    }
 
-            sel.load::<(
-                    models::Job,
-                    models::Submit,
-                    models::Endpoint,
-                    models::Package,
-                    Option<(models::JobEnv, models::EnvVar)>,
-                )>(&conn)
-                .map_err(Error::from)
-        })
-        .unwrap_or_else(|| {
-            let sel = dsl::jobs
-                .inner_join(crate::schema::submits::table)
-                .inner_join(crate::schema::endpoints::table)
-                .inner_join(crate::schema::packages::table)
-                .left_outer_join(schema::job_envs::table.inner_join(schema::envvars::table))
-                .into_boxed();
+    if let Some(datetime) = older_than_filter.as_ref() {
+        sel = sel.filter(schema::submits::dsl::submit_time.lt(datetime))
+    }
 
-            let sel = if let Some((name, val)) = env_filter_tpl.as_ref() {
-                use crate::diesel::BoolExpressionMethods;
+    if let Some(datetime) = newer_than_filter.as_ref() {
+        sel = sel.filter(schema::submits::dsl::submit_time.gt(datetime))
+    }
 
-                sel.filter({
-                    schema::envvars::dsl::name.eq(name.as_ref())
-                        .and(schema::envvars::dsl::value.eq(val))
-                })
-            } else {
-                sel
-            };
+    if let Some(limit) = matches.value_of("limit").map(i64::from_str).transpose()? {
+        sel = sel.limit(limit)
+    }
 
-            sel.load::<(
-                    models::Job,
-                    models::Submit,
-                    models::Endpoint,
-                    models::Package,
-                    Option<(models::JobEnv, models::EnvVar)>,
-                )>(&conn)
-                .map_err(Error::from)
-        })?;
+    if let Some(ep_name) = matches.value_of("endpoint") {
+        sel = sel.filter(schema::endpoints::name.eq(ep_name))
+    }
 
-    let data = jobs
-        .iter()
-        .group_by(|(job, submit, ep, package, _)| {
-            (job, submit, ep, package)
-        });
+    if let Some(pkg_name) = matches.value_of("package") {
+        sel = sel.filter(schema::packages::name.eq(pkg_name))
+    }
 
-    let data = data
+    let data = sel
+        .order_by(schema::jobs::id.desc()) // required for the --limit implementation
+        .load::<(models::Job, models::Submit, models::Endpoint, models::Package)>(&conn)?
         .into_iter()
-        .map(|((job, submit, ep, package), grouped)| {
-            let envs = grouped
-                .filter_map(|opt| opt.4.as_ref())
-                .map(|tpl| format!("{}={}", tpl.1.name, tpl.1.value))
-                .join(", ");
-
-            let success = crate::log::ParsedLog::build_from(&job.log_text)?
-                .is_successfull()
+        .rev() // required for the --limit implementation
+        .map(|(job, submit, ep, package)| {
+            let success = is_job_successfull(&job)?
                 .map(|b| if b { "yes" } else { "no" })
                 .map(String::from)
                 .unwrap_or_else(|| String::from("unknown"));
 
             Ok(vec![
-                format!("{}", job.id),
                 submit.uuid.to_string(),
                 job.uuid.to_string(),
                 submit.submit_time.to_string(),
-                ep.name.clone(),
+                ep.name,
                 success,
-                package.name.clone(),
-                package.version.clone(),
-                envs,
+                package.name,
+                package.version,
             ])
         })
         .collect::<Result<Vec<_>>>()?;
@@ -415,14 +494,15 @@ fn jobs(conn_cfg: DbConnectionConfig, matches: &ArgMatches) -> Result<()> {
     Ok(())
 }
 
-fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the "db job" subcommand
+fn job(conn_cfg: DbConnectionConfig<'_>, config: &Configuration, matches: &ArgMatches) -> Result<()> {
     let script_highlight = !matches.is_present("no_script_highlight");
     let script_line_numbers = !matches.is_present("no_script_line_numbers");
     let configured_theme = config.script_highlight_theme();
     let show_log = matches.is_present("show_log");
     let show_script = matches.is_present("show_script");
     let csv = matches.is_present("csv");
-    let conn = crate::db::establish_connection(conn_cfg)?;
+    let conn = conn_cfg.establish_connection()?;
     let job_uuid = matches
         .value_of("job_uuid")
         .map(uuid::Uuid::parse_str)
@@ -443,13 +523,16 @@ fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatche
             models::Image,
         )>(&conn)?;
 
-    let parsed_log = crate::log::ParsedLog::build_from(&data.0.log_text)?;
+    trace!("Parsing log");
+    let parsed_log = crate::log::ParsedLog::from_str(&data.0.log_text)?;
+    trace!("Parsed log = {:?}", parsed_log);
     let success = parsed_log.is_successfull();
+    trace!("log successfull = {:?}", success);
 
     if csv {
         let hdrs = crate::commands::util::mk_header(vec![
             "UUID",
-            "success",
+            "Success",
             "Package Name",
             "Package Version",
             "Ran on",
@@ -460,9 +543,9 @@ fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatche
         let data = vec![vec![
             data.0.uuid.to_string(),
             String::from(match success {
-                Some(true) => "yes",
-                Some(false) => "no",
-                None => "unknown",
+                JobResult::Success => "yes",
+                JobResult::Errored => "no",
+                JobResult::Unknown => "unknown",
             }),
             data.3.name.to_string(),
             data.3.version.to_string(),
@@ -504,15 +587,15 @@ fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatche
 
             "#,
             job_uuid = match success {
-                Some(true) => data.0.uuid.to_string().green(),
-                Some(false) => data.0.uuid.to_string().red(),
-                None => data.0.uuid.to_string().cyan(),
+                JobResult::Success => data.0.uuid.to_string().green(),
+                JobResult::Errored => data.0.uuid.to_string().red(),
+                JobResult::Unknown => data.0.uuid.to_string().cyan(),
             },
             submit_uuid = data.1.uuid.to_string().cyan(),
             succeeded = match success {
-                Some(true) => String::from("yes").green(),
-                Some(false) => String::from("no").red(),
-                None => String::from("unknown").cyan(),
+                JobResult::Success => String::from("yes").green(),
+                JobResult::Errored => String::from("no").red(),
+                JobResult::Unknown => String::from("unknown").cyan(),
             },
             package_name = data.3.name.cyan(),
             package_version = data.3.version.cyan(),
@@ -563,14 +646,8 @@ fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatche
 
         if show_log {
             let log = parsed_log
-                .iter()
-                .map(|line_item| match line_item {
-                    LogItem::Line(s) => Ok(String::from_utf8(s.to_vec())?.normal()),
-                    LogItem::Progress(u) => Ok(format!("#BUTIDO:PROGRESS:{}", u).bright_black()),
-                    LogItem::CurrentPhase(p) => Ok(format!("#BUTIDO:PHASE:{}", p).bright_black()),
-                    LogItem::State(Ok(())) => Ok("#BUTIDO:STATE:OK".to_string().green()),
-                    LogItem::State(Err(s)) => Ok(format!("#BUTIDO:STATE:ERR:{}", s).red()),
-                })
+                .into_iter()
+                .map(|line_item| line_item.display().map(|d| d.to_string()))
                 .collect::<Result<Vec<_>>>()?
                 .into_iter() // ugly, but hey... not important right now.
                 .join("\n");
@@ -591,11 +668,35 @@ fn job(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatche
     }
 }
 
-fn releases(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgMatches) -> Result<()> {
+/// Implementation of the subcommand "db log-of"
+fn log_of(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
+    let conn   = conn_cfg.establish_connection()?;
+    let job_uuid = matches
+        .value_of("job_uuid")
+        .map(uuid::Uuid::parse_str)
+        .transpose()?
+        .unwrap();
+    let out = std::io::stdout();
+    let mut lock = out.lock();
+
+    schema::jobs::table
+        .filter(schema::jobs::dsl::uuid.eq(job_uuid))
+        .select(schema::jobs::dsl::log_text)
+        .first::<String>(&conn)
+        .map_err(Error::from)
+        .and_then(|s| crate::log::ParsedLog::from_str(&s))?
+        .into_iter()
+        .map(|line| line.display().and_then(|d| writeln!(lock, "{}", d).map_err(Error::from)))
+        .collect::<Result<Vec<()>>>()
+        .map(|_| ())
+}
+
+/// Implementation of the "db releases" subcommand
+fn releases(conn_cfg: DbConnectionConfig<'_>, config: &Configuration, matches: &ArgMatches) -> Result<()> {
     let csv    = matches.is_present("csv");
-    let conn   = crate::db::establish_connection(conn_cfg)?;
+    let conn   = conn_cfg.establish_connection()?;
     let header = crate::commands::util::mk_header(["Package", "Version", "Date", "Path"].to_vec());
-    let data   = schema::jobs::table
+    let mut query = schema::jobs::table
         .inner_join(schema::packages::table)
         .inner_join(schema::artifacts::table)
         .inner_join(schema::releases::table
@@ -605,6 +706,25 @@ fn releases(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgM
         .order_by(schema::packages::dsl::name.asc())
         .then_order_by(schema::packages::dsl::version.asc())
         .then_order_by(schema::releases::release_date.asc())
+        .into_boxed();
+
+    if let Some(date) = crate::commands::util::get_date_filter("older_than", matches)? {
+        query = query.filter(schema::releases::release_date.lt(date));
+    }
+
+    if let Some(date) = crate::commands::util::get_date_filter("newer_than", matches)? {
+        query = query.filter(schema::releases::release_date.gt(date));
+    }
+
+    if let Some(store) = matches.value_of("store") {
+        query = query.filter(schema::release_stores::dsl::store_name.eq(store));
+    }
+
+    if let Some(pkg) = matches.value_of("package") {
+        query = query.filter(schema::packages::dsl::name.eq(pkg));
+    }
+
+    let data = query
         .select({
             let art = schema::artifacts::all_columns;
             let pac = schema::packages::all_columns;
@@ -634,3 +754,10 @@ fn releases(conn_cfg: DbConnectionConfig, config: &Configuration, matches: &ArgM
     crate::commands::util::display_data(header, data, csv)
 }
 
+/// Check if a job is successful
+///
+/// Returns Ok(None) if cannot be decided
+fn is_job_successfull(job: &models::Job) -> Result<Option<bool>> {
+    crate::log::ParsedLog::from_str(&job.log_text).map(|pl| pl.is_successfull().to_bool())
+}
+
