@@ -544,7 +544,7 @@ impl<'a> PreparedContainer<'a> {
             .iter()
             .map(|patch| async move {
                 let destination = PathBuf::from(crate::consts::PATCH_DIR_PATH).join(patch);
-                trace!("Copying patch {} to container at {}/{}", crate::consts::PATCH_DIR_PATH, patch.display(), destination.display());
+                trace!("Copying patch {} to container at {}", patch.display(), destination.display());
 
                 let mut buf = vec![];
                 tokio::fs::OpenOptions::new()
